@@ -229,11 +229,11 @@ attachment_button_update_pixbufs (EAttachmentButton *button)
 
 	pixbuf_expander_open = gtk_icon_theme_load_icon (
 		icon_theme, "go-down",
-		GTK_ICON_SIZE_BUTTON, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+		GTK_ICON_SIZE_BUTTON, 0, NULL);
 
 	pixbuf_expander_closed = gtk_icon_theme_load_icon (
 		icon_theme, "go-next",
-		GTK_ICON_SIZE_BUTTON, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+		GTK_ICON_SIZE_BUTTON, 0, NULL);
 
 	g_object_set (
 		renderer,
@@ -681,7 +681,7 @@ e_attachment_button_init (EAttachmentButton *button)
 }
 
 GtkWidget *
-e_attachment_button_new ()
+e_attachment_button_new (void)
 {
 	return g_object_new (
 		E_TYPE_ATTACHMENT_BUTTON, NULL);
