@@ -43,6 +43,8 @@ struct _EHTMLEditorPageDialogPrivate {
 	GtkWidget *background_image_filechooser;
 
 	GtkWidget *remove_image_button;
+
+	EHTMLEditorViewHistoryEvent *history_event;
 };
 
 typedef struct _Template {
@@ -189,6 +191,7 @@ html_editor_page_dialog_set_link_color (EHTMLEditorPageDialog *dialog)
 	color = g_strdup_printf ("#%06x", e_rgba_to_value (&rgba));
 	webkit_dom_html_body_element_set_link (
 		WEBKIT_DOM_HTML_BODY_ELEMENT (body), color);
+	e_html_editor_view_set_link_color (view, &rgba);
 
 	g_free (color);
 }
@@ -328,6 +331,20 @@ html_editor_page_dialog_show (GtkWidget *widget)
 	document = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (view));
 	body = webkit_dom_document_get_body (document);
 
+	if (!e_html_editor_view_is_undo_redo_in_progress (view)) {
+		EHTMLEditorSelection *selection;
+		EHTMLEditorViewHistoryEvent *ev;
+
+		ev = g_new0 (EHTMLEditorViewHistoryEvent, 1);
+		ev->type = HISTORY_PAGE_DIALOG;
+
+		selection = e_html_editor_view_get_selection (view);
+		e_html_editor_selection_get_selection_coordinates (
+			selection, &ev->before.start.x, &ev->before.start.y, &ev->before.end.x, &ev->before.end.y);
+		ev->data.dom.from = webkit_dom_node_clone_node (WEBKIT_DOM_NODE (body), FALSE);
+		dialog->priv->history_event = ev;
+	}
+
 	tmp = webkit_dom_element_get_attribute (
 		WEBKIT_DOM_ELEMENT (body), "data-uri");
 	if (tmp && *tmp) {
@@ -361,14 +378,24 @@ html_editor_page_dialog_show (GtkWidget *widget)
 	tmp = webkit_dom_html_body_element_get_link (
 			WEBKIT_DOM_HTML_BODY_ELEMENT (body));
 	if (!tmp || !*tmp) {
-		GdkColor color;
-		gtk_widget_style_get (
-			GTK_WIDGET (view), "link-color", &color, NULL);
+		GdkColor *color = NULL;
+		GtkStyleContext *context;
 
-		rgba.alpha = 1;
-		rgba.red = ((gdouble) color.red) / G_MAXUINT16;
-		rgba.green = ((gdouble) color.green) / G_MAXUINT16;
-		rgba.blue = ((gdouble) color.blue) / G_MAXUINT16;
+		context = gtk_widget_get_style_context (GTK_WIDGET (view));
+		gtk_style_context_get_style (
+			context, "link-color", &color, NULL);
+
+		if (color == NULL) {
+			rgba.alpha = 1;
+			rgba.red = 0;
+			rgba.green = 0;
+			rgba.blue = 1;
+		} else {
+			rgba.alpha = 1;
+			rgba.red = ((gdouble) color->red) / G_MAXUINT16;
+			rgba.green = ((gdouble) color->green) / G_MAXUINT16;
+			rgba.blue = ((gdouble) color->blue) / G_MAXUINT16;
+		}
 	} else {
 		gdk_rgba_parse (&rgba, tmp);
 	}
@@ -387,6 +414,41 @@ html_editor_page_dialog_show (GtkWidget *widget)
 	GTK_WIDGET_CLASS (e_html_editor_page_dialog_parent_class)->show (widget);
 }
 
+static void
+html_editor_page_dialog_hide (GtkWidget *widget)
+{
+	EHTMLEditorPageDialogPrivate *priv;
+	EHTMLEditorViewHistoryEvent *ev;
+
+	priv = E_HTML_EDITOR_PAGE_DIALOG_GET_PRIVATE (widget);
+	ev = priv->history_event;
+
+	if (ev) {
+		EHTMLEditorPageDialog *dialog;
+		EHTMLEditor *editor;
+		EHTMLEditorSelection *selection;
+		EHTMLEditorView *view;
+		WebKitDOMDocument *document;
+		WebKitDOMHTMLElement *body;
+
+		dialog = E_HTML_EDITOR_PAGE_DIALOG (widget);
+		editor = e_html_editor_dialog_get_editor (E_HTML_EDITOR_DIALOG (dialog));
+		view = e_html_editor_get_view (editor);
+		selection = e_html_editor_view_get_selection (view);
+
+		document = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (view));
+		body = webkit_dom_document_get_body (document);
+
+		ev->data.dom.to = webkit_dom_node_clone_node (WEBKIT_DOM_NODE (body), FALSE);
+
+		e_html_editor_selection_get_selection_coordinates (
+			selection, &ev->after.start.x, &ev->after.start.y, &ev->after.end.x, &ev->after.end.y);
+		e_html_editor_view_insert_new_history_event (view, ev);
+	}
+
+	GTK_WIDGET_CLASS (e_html_editor_page_dialog_parent_class)->hide (widget);
+}
+
 static void
 e_html_editor_page_dialog_class_init (EHTMLEditorPageDialogClass *class)
 {
@@ -396,6 +458,7 @@ e_html_editor_page_dialog_class_init (EHTMLEditorPageDialogClass *class)
 
 	widget_class = GTK_WIDGET_CLASS (class);
 	widget_class->show = html_editor_page_dialog_show;
+	widget_class->hide = html_editor_page_dialog_hide;
 }
 
 static void
@@ -533,6 +596,6 @@ e_html_editor_page_dialog_new (EHTMLEditor *editor)
 		g_object_new (
 			E_TYPE_HTML_EDITOR_PAGE_DIALOG,
 			"editor", editor,
-			"title", N_("Page Properties"),
+			"title", _("Page Properties"),
 			NULL));
 }
