@@ -57,6 +57,8 @@ struct _EHTMLEditorTableDialogPrivate {
 	GtkWidget *remove_image_button;
 
 	WebKitDOMHTMLTableElement *table_element;
+
+	EHTMLEditorViewHistoryEvent *history_event;
 };
 
 static GdkRGBA transparent = { 0, 0, 0, 0 };
@@ -72,16 +74,18 @@ html_editor_table_dialog_create_table (EHTMLEditorTableDialog *dialog)
 	EHTMLEditor *editor;
 	EHTMLEditorSelection *editor_selection;
 	EHTMLEditorView *view;
-	WebKitDOMDocument *document;
-	WebKitDOMElement *table, *br, *caret, *parent, *element;
 	gint i;
+	gchar *text_content;
+	gboolean empty = FALSE;
+	WebKitDOMDocument *document;
+	WebKitDOMElement *table, *br, *caret, *element, *cell;
+	WebKitDOMNode *clone;
 
 	editor = e_html_editor_dialog_get_editor (E_HTML_EDITOR_DIALOG (dialog));
 	view = e_html_editor_get_view (editor);
 	editor_selection = e_html_editor_view_get_selection (view);
 
-	document = webkit_web_view_get_dom_document (
-		WEBKIT_WEB_VIEW (view));
+	document = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (view));
 
 	/* Default 3x3 table */
 	table = webkit_dom_document_create_element (document, "TABLE", NULL);
@@ -98,32 +102,53 @@ html_editor_table_dialog_create_table (EHTMLEditorTableDialog *dialog)
 		}
 	}
 
-	caret = e_html_editor_selection_save_caret_position (editor_selection);
+	e_html_editor_selection_save (editor_selection);
+	caret = webkit_dom_document_get_element_by_id (
+		document, "-x-evo-selection-end-marker");
 
-	parent = webkit_dom_node_get_parent_element (WEBKIT_DOM_NODE (caret));
-	element = caret;
+	element = get_parent_block_element (WEBKIT_DOM_NODE (caret));
+	text_content = webkit_dom_node_get_text_content (WEBKIT_DOM_NODE (element));
+	empty = text_content && !*text_content;
+	g_free (text_content);
 
-	while (!WEBKIT_DOM_IS_HTML_BODY_ELEMENT (parent)) {
-		element = parent;
-		parent = webkit_dom_node_get_parent_element (
-			WEBKIT_DOM_NODE (parent));
+	clone = webkit_dom_node_clone_node (WEBKIT_DOM_NODE (element), FALSE);
+	br = webkit_dom_document_create_element (document, "BR", NULL);
+	webkit_dom_node_append_child (clone, WEBKIT_DOM_NODE (br), NULL);
+	webkit_dom_node_insert_before (
+		webkit_dom_node_get_parent_node (WEBKIT_DOM_NODE (element)),
+		clone,
+		webkit_dom_node_get_next_sibling (WEBKIT_DOM_NODE (element)),
+		NULL);
+
+	/* Move caret to the first cell */
+	cell = webkit_dom_element_query_selector (table, "td", NULL);
+	webkit_dom_node_append_child (
+		WEBKIT_DOM_NODE (cell), WEBKIT_DOM_NODE (caret), NULL);
+	caret = webkit_dom_document_get_element_by_id (
+		document, "-x-evo-selection-start-marker");
+	webkit_dom_node_insert_before (
+		WEBKIT_DOM_NODE (cell),
+		WEBKIT_DOM_NODE (caret),
+		webkit_dom_node_get_last_child (WEBKIT_DOM_NODE (cell)),
+		NULL);
+
+	/* Insert the table into body unred the current block (if current block is not empty)
+	 * otherwise replace the current block. */
+	if (empty) {
+		webkit_dom_node_replace_child (
+			webkit_dom_node_get_parent_node (WEBKIT_DOM_NODE (element)),
+			WEBKIT_DOM_NODE (table),
+			WEBKIT_DOM_NODE (element),
+			NULL);
+	} else {
+		webkit_dom_node_insert_before (
+			webkit_dom_node_get_parent_node (WEBKIT_DOM_NODE (element)),
+			WEBKIT_DOM_NODE (table),
+			webkit_dom_node_get_next_sibling (WEBKIT_DOM_NODE (element)),
+			NULL);
 	}
 
-	br = webkit_dom_document_create_element (document, "BR", NULL);
-	webkit_dom_node_insert_before (
-		WEBKIT_DOM_NODE (parent),
-		WEBKIT_DOM_NODE (br),
-		webkit_dom_node_get_next_sibling (WEBKIT_DOM_NODE (element)),
-		NULL);
-
-	/* Insert the table into body below the caret */
-	webkit_dom_node_insert_before (
-		WEBKIT_DOM_NODE (parent),
-		WEBKIT_DOM_NODE (table),
-		webkit_dom_node_get_next_sibling (WEBKIT_DOM_NODE (element)),
-		NULL);
-
-	e_html_editor_selection_clear_caret_position_marker (editor_selection);
+	e_html_editor_selection_restore (editor_selection);
 
 	e_html_editor_view_set_changed (view, TRUE);
 
@@ -154,6 +179,7 @@ html_editor_table_dialog_set_row_count (EHTMLEditorTableDialog *dialog)
 				dialog->priv->table_element, -1, NULL);
 		}
 	}
+	g_object_unref (rows);
 }
 
 static void
@@ -168,6 +194,7 @@ html_editor_table_dialog_get_row_count (EHTMLEditorTableDialog *dialog)
 	gtk_spin_button_set_value (
 		GTK_SPIN_BUTTON (dialog->priv->rows_edit),
 		webkit_dom_html_collection_get_length (rows));
+	g_object_unref (rows);
 }
 
 static void
@@ -205,7 +232,10 @@ html_editor_table_dialog_set_column_count (EHTMLEditorTableDialog *dialog)
 					row, -1, NULL);
 			}
 		}
+		g_object_unref (row);
+		g_object_unref (cells);
 	}
+	g_object_unref (rows);
 }
 
 static void
@@ -225,6 +255,9 @@ html_editor_table_dialog_get_column_count (EHTMLEditorTableDialog *dialog)
 	gtk_spin_button_set_value (
 		GTK_SPIN_BUTTON (dialog->priv->columns_edit),
 		webkit_dom_html_collection_get_length (columns));
+	g_object_unref (row);
+	g_object_unref (rows);
+	g_object_unref (columns);
 }
 
 static void
@@ -626,6 +659,22 @@ html_editor_table_dialog_show (GtkWidget *widget)
 				WEBKIT_DOM_HTML_TABLE_ELEMENT (table);
 			html_editor_table_dialog_get_values (dialog);
 		}
+
+		if (!e_html_editor_view_is_undo_redo_in_progress (view)) {
+			EHTMLEditorViewHistoryEvent *ev;
+
+			ev = g_new0 (EHTMLEditorViewHistoryEvent, 1);
+			ev->type = HISTORY_TABLE_DIALOG;
+
+			e_html_editor_selection_get_selection_coordinates (
+				e_html_editor_view_get_selection (view),
+				&ev->before.start.x, &ev->before.start.y,
+				&ev->before.end.x, &ev->before.end.y);
+			if (table)
+				ev->data.dom.from = webkit_dom_node_clone_node (
+					WEBKIT_DOM_NODE (table), TRUE);
+			dialog->priv->history_event = ev;
+		}
 	}
 
 	/* Chain up to parent implementation */
@@ -648,9 +697,31 @@ static void
 html_editor_table_dialog_hide (GtkWidget *widget)
 {
 	EHTMLEditorTableDialogPrivate *priv;
+	EHTMLEditorViewHistoryEvent *ev;
 
 	priv = E_HTML_EDITOR_TABLE_DIALOG_GET_PRIVATE (widget);
+	ev = priv->history_event;
 
+	if (ev) {
+		EHTMLEditorTableDialog *dialog;
+		EHTMLEditor *editor;
+		EHTMLEditorSelection *selection;
+		EHTMLEditorView *view;
+
+		dialog = E_HTML_EDITOR_TABLE_DIALOG (widget);
+		editor = e_html_editor_dialog_get_editor (E_HTML_EDITOR_DIALOG (dialog));
+		view = e_html_editor_get_view (editor);
+		selection = e_html_editor_view_get_selection (view);
+
+		ev->data.dom.to = webkit_dom_node_clone_node (
+			WEBKIT_DOM_NODE (priv->table_element), TRUE);
+
+		e_html_editor_selection_get_selection_coordinates (
+			selection, &ev->after.start.x, &ev->after.start.y, &ev->after.end.x, &ev->after.end.y);
+		e_html_editor_view_insert_new_history_event (view, ev);
+	}
+
+	g_object_unref (priv->table_element);
 	priv->table_element = NULL;
 
 	GTK_WIDGET_CLASS (e_html_editor_table_dialog_parent_class)->hide (widget);
@@ -906,7 +977,7 @@ e_html_editor_table_dialog_new (EHTMLEditor *editor)
 		g_object_new (
 			E_TYPE_HTML_EDITOR_TABLE_DIALOG,
 			"editor", editor,
-			"title", N_("Table Properties"),
+			"title", _("Table Properties"),
 			NULL));
 }
 
