@@ -384,7 +384,7 @@ bogofilter_new_config_widget (EMailJunkFilter *junk_filter)
 	gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 0);
 	gtk_widget_show (widget);
 
-	g_object_bind_property (
+	e_binding_bind_property (
 		junk_filter, "convert-to-unicode",
 		widget, "active",
 		G_BINDING_BIDIRECTIONAL |
