@@ -1,18 +1,17 @@
 /*
  * e-contacts-selector.c
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) version 3.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -45,7 +44,7 @@ contacts_selector_get_source_selected (ESourceSelector *selector,
 	return e_source_contacts_get_include_me (extension);
 }
 
-static void
+static gboolean
 contacts_selector_set_source_selected (ESourceSelector *selector,
                                        ESource *source,
                                        gboolean selected)
@@ -56,16 +55,20 @@ contacts_selector_set_source_selected (ESourceSelector *selector,
 	/* Make sure this source is an address book. */
 	extension_name = e_source_selector_get_extension_name (selector);
 	if (!e_source_has_extension (source, extension_name))
-		return;
+		return FALSE;
 
 	extension_name = E_SOURCE_EXTENSION_CONTACTS_BACKEND;
 	extension = e_source_get_extension (source, extension_name);
-	g_return_if_fail (E_IS_SOURCE_CONTACTS (extension));
+	g_return_val_if_fail (E_IS_SOURCE_CONTACTS (extension), FALSE);
 
 	if (selected != e_source_contacts_get_include_me (extension)) {
 		e_source_contacts_set_include_me (extension, selected);
 		e_source_selector_queue_write (selector, source);
+
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
 static void
