@@ -294,6 +294,21 @@ shell_sidebar_size_allocate (GtkWidget *widget,
 		gtk_widget_size_allocate (child, &child_allocation);
 }
 
+static void
+shell_sidebar_style_updated (GtkWidget *widget)
+{
+	EShellSidebar *shell_sidebar;
+	GdkRGBA bg;
+
+	/* Chain up to parent's method. */
+	GTK_WIDGET_CLASS (e_shell_sidebar_parent_class)->style_updated (widget);
+
+	shell_sidebar = E_SHELL_SIDEBAR (widget);
+
+	e_utils_get_theme_color (shell_sidebar->priv->event_box, "theme_bg_color", E_UTILS_DEFAULT_THEME_BG_COLOR, &bg);
+	gtk_widget_override_background_color (shell_sidebar->priv->event_box, GTK_STATE_FLAG_NORMAL, &bg);
+}
+
 static void
 shell_sidebar_forall (GtkContainer *container,
                       gboolean include_internals,
@@ -351,6 +366,7 @@ e_shell_sidebar_class_init (EShellSidebarClass *class)
 	widget_class->get_preferred_width = shell_sidebar_get_preferred_width;
 	widget_class->get_preferred_height = shell_sidebar_get_preferred_height;
 	widget_class->size_allocate = shell_sidebar_size_allocate;
+	widget_class->style_updated = shell_sidebar_style_updated;
 
 	container_class = GTK_CONTAINER_CLASS (class);
 	container_class->forall = shell_sidebar_forall;
@@ -431,12 +447,11 @@ e_shell_sidebar_alert_sink_init (EAlertSinkInterface *iface)
 static void
 e_shell_sidebar_init (EShellSidebar *shell_sidebar)
 {
-	GtkStyle *style;
 	GtkWidget *widget;
 	GtkWidget *container;
 	PangoAttribute *attribute;
 	PangoAttrList *attribute_list;
-	const GdkColor *color;
+	GdkRGBA bg;
 	const gchar *icon_name;
 
 	shell_sidebar->priv = E_SHELL_SIDEBAR_GET_PRIVATE (shell_sidebar);
@@ -444,9 +459,8 @@ e_shell_sidebar_init (EShellSidebar *shell_sidebar)
 	gtk_widget_set_has_window (GTK_WIDGET (shell_sidebar), FALSE);
 
 	widget = gtk_event_box_new ();
-	style = gtk_widget_get_style (widget);
-	color = &style->bg[GTK_STATE_ACTIVE];
-	gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, color);
+	e_utils_get_theme_color (widget, "theme_bg_color", E_UTILS_DEFAULT_THEME_BG_COLOR, &bg);
+	gtk_widget_override_background_color (widget, GTK_STATE_FLAG_NORMAL, &bg);
 	gtk_widget_set_parent (widget, GTK_WIDGET (shell_sidebar));
 	shell_sidebar->priv->event_box = g_object_ref (widget);
 	gtk_widget_show (widget);
@@ -467,7 +481,7 @@ e_shell_sidebar_init (EShellSidebar *shell_sidebar)
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
 	gtk_widget_show (widget);
 
-	g_object_bind_property (
+	e_binding_bind_property (
 		shell_sidebar, "icon-name",
 		widget, "icon-name",
 		G_BINDING_SYNC_CREATE);
@@ -484,7 +498,7 @@ e_shell_sidebar_init (EShellSidebar *shell_sidebar)
 	gtk_label_set_attributes (GTK_LABEL (widget), attribute_list);
 	pango_attr_list_unref (attribute_list);
 
-	g_object_bind_property (
+	e_binding_bind_property (
 		shell_sidebar, "primary-text",
 		widget, "label",
 		G_BINDING_SYNC_CREATE);
@@ -500,7 +514,7 @@ e_shell_sidebar_init (EShellSidebar *shell_sidebar)
 	gtk_label_set_attributes (GTK_LABEL (widget), attribute_list);
 	pango_attr_list_unref (attribute_list);
 
-	g_object_bind_property (
+	e_binding_bind_property (
 		shell_sidebar, "secondary-text",
 		widget, "label",
 		G_BINDING_SYNC_CREATE);
